@@ -182,6 +182,12 @@ class LDAPObject(RecordableMethods):
         """
         return "dn:" + self.bound_as
 
+    @recorded
+    def passwd_s(self, user, oldpw, newpw, serverctrls=None, clientctrls=None):
+        """
+        """
+        return self._passwd_s(user, oldpw, newpw, serverctrls, clientctrls)
+
     #
     # Internal implementations
     #
@@ -242,7 +248,7 @@ class LDAPObject(RecordableMethods):
         # Find directory entries within the requested scope
         base_parts = ldap.dn.explode_dn(base.lower())
         base_len = len(base_parts)
-        dn_parts = dict((dn, ldap.dn.explode_dn(dn)) for dn in self.directory.keys())
+        dn_parts = dict((dn, ldap.dn.explode_dn(dn.lower())) for dn in self.directory.keys())
 
         if scope == ldap.SCOPE_BASE:
             dns = (dn for dn, parts in dn_parts.items() if parts == base_parts)
@@ -328,7 +334,7 @@ class LDAPObject(RecordableMethods):
             self.directory[dn]
             raise ldap.ALREADY_EXISTS
         except KeyError:
-            self.directory[dn] = entry
+            self.directory[dn.lower()] = entry
             return (105, [], len(self.methods_called()), [])
 
     def _rename_s(self, dn, newrdn, newsuperior):
@@ -380,6 +386,19 @@ class LDAPObject(RecordableMethods):
 
         return (107, [])
 
+    def _passwd_s(self, user, oldpw, newpw, serverctrls=None, clientctrls=None):
+        self._check_valid_dn(user)
+
+        entry = self.directory.get(user)
+        if entry is None:
+            raise ldap.NO_SUCH_OBJECT
+
+        if oldpw is not None:
+            if entry['userPassword'][0] == oldpw:
+                entry['userPassword'] = [newpw]
+        else:
+            entry['userPassword'] = [newpw]
+
     #
     # Async
     #
